/* $Id: vmdetlin.c,v 1.3 2009/02/17 14:43:55 cizzo Exp $
 *
 * This file is part of the VIMOS Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: cizzo $
 * $Date: 2009/02/17 14:43:55 $
 * $Revision: 1.3 $
 * $Name: vimosp-2_6_0 $
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*----------------------------------------------------------------------------
                             Includes
 ----------------------------------------------------------------------------*/

#include <cpl.h>

#include "irplib_detmon.h"
#include "irplib_detmon_lg.h"


/* Define here the DO.CATG keywords */
#define DETMON_OPT_LG_ON_RAW                    "DETECTOR_CHECK_FLAT"
#define DETMON_OPT_LG_OFF_RAW                        "DETECTOR_CHECK_BIAS"

#define RECIPE_NAME "vmdetlin"

#define OPT FALSE

/*----------------------------------------------------------------------------
                                Functions prototypes
 ----------------------------------------------------------------------------*/

static cpl_error_code
vmdetlin_fill_parlist(cpl_parameterlist *);

CPL_RECIPE_DEFINE(vmdetlin, VIMOS_BINARY_VERSION,
                  vmdetlin_fill_parlist(recipe->parameters),
                  "Lander de Bilbao", "lbilbao@eso.org", "2008", 
                  "Linearity/Gain recipe for the optical domain",
                  irplib_detmon_lg_get_description("vmdetlin", 
                                                   PACKAGE_TARNAME,
                                                   DETMON_OPT_LG_ON_RAW,
                                                   DETMON_OPT_LG_OFF_RAW));

/*---------------------------------------------------------------------------*/
/*
  @brief    Interpret the command line options and execute the data processing
  @param    frameset    the frames list
  @param    parlist     the parameters list
  @return   0 if everything is ok
 */
/*---------------------------------------------------------------------------*/

static int vmdetlin(cpl_frameset            *frameset,
                    const cpl_parameterlist *parlist)
{
    cpl_propertylist * lintbl = 
        irplib_detmon_fill_prolist("DET_LIN_INFO", NULL, NULL, CPL_FALSE);

    cpl_propertylist * gaintbl = 
        irplib_detmon_fill_prolist("GAIN_INFO",    NULL, NULL, CPL_FALSE);

    cpl_propertylist * coeffscube = 
        irplib_detmon_fill_prolist("COEFFS_CUBE",  NULL, NULL, CPL_FALSE);

    cpl_propertylist * bpm = 
        irplib_detmon_fill_prolist("BP_MAP_NL",    NULL, NULL, CPL_FALSE);

    cpl_propertylist * corr = 
        irplib_detmon_fill_prolist("AUTOCORR",     NULL, NULL, CPL_FALSE);

    cpl_propertylist * diff_flat = 
        irplib_detmon_fill_prolist("DIFF_FLAT",    NULL, NULL, CPL_FALSE);

    const cpl_error_code error = irplib_detmon_lg(frameset,
                                                  parlist,
                                                  DETMON_OPT_LG_ON_RAW,
                                                  DETMON_OPT_LG_OFF_RAW,
                                                  RECIPE_NAME,
                                                  PACKAGE_TARNAME,
                                                  REGEXP,
                                                  lintbl, gaintbl, coeffscube,
                                                  bpm, corr, diff_flat,
                                                  PACKAGE "/" PACKAGE_VERSION,
                                                  NULL, NULL, OPT);

    cpl_propertylist_delete(lintbl);
    cpl_propertylist_delete(gaintbl);
    cpl_propertylist_delete(coeffscube);
    cpl_propertylist_delete(bpm);
    cpl_propertylist_delete(corr);
    cpl_propertylist_delete(diff_flat);

    /* Propagate the error, if any */
    cpl_ensure_code(!error, error);

    return CPL_ERROR_NONE;
}

static cpl_error_code
vmdetlin_fill_parlist(cpl_parameterlist * parlist)
{
    cpl_error_code error;
    cpl_parameter * p;

    error = irplib_detmon_lg_fill_parlist_opt_default(parlist,
                                              "vmdetlin", PACKAGE_TARNAME);

    cpl_ensure_code(!error, error);

 
    p = cpl_parameterlist_find(parlist,
                               PACKAGE_TARNAME "." RECIPE_NAME ".tolerance");
    cpl_ensure_code(p != NULL, CPL_ERROR_DATA_NOT_FOUND);
    error = cpl_parameter_set_default_double(p, 0.1);
    cpl_ensure_code(!error, error);

    p = cpl_parameterlist_find(parlist,
                               PACKAGE_TARNAME "." RECIPE_NAME ".pix2pix");
    cpl_ensure_code(p != NULL, CPL_ERROR_DATA_NOT_FOUND);
    error = cpl_parameter_set_default_bool(p, 1);
    cpl_ensure_code(!error, error);

    p = cpl_parameterlist_find(parlist,
                               PACKAGE_TARNAME "." RECIPE_NAME ".order");
    cpl_ensure_code(p != NULL, CPL_ERROR_DATA_NOT_FOUND);
    error = cpl_parameter_set_default_int(p, 2);
    cpl_ensure_code(!error, error);

    p = cpl_parameterlist_find(parlist,
                               PACKAGE_TARNAME "." RECIPE_NAME ".kappa");
    cpl_ensure_code(p != NULL, CPL_ERROR_DATA_NOT_FOUND);
    error = cpl_parameter_set_default_double(p, 4.0);
    cpl_ensure_code(!error, error);

    p = cpl_parameterlist_find(parlist,
                               PACKAGE_TARNAME "." RECIPE_NAME ".llx");
    cpl_ensure_code(p != NULL, CPL_ERROR_DATA_NOT_FOUND);
    error = cpl_parameter_set_default_int(p, 51);
    cpl_ensure_code(!error, error);

    p = cpl_parameterlist_find(parlist,
                               PACKAGE_TARNAME "." RECIPE_NAME ".urx");
    cpl_ensure_code(p != NULL, CPL_ERROR_DATA_NOT_FOUND);
    error = cpl_parameter_set_default_int(p, 2098);
    cpl_ensure_code(!error, error);

    return error;

}
